@@ -38,6 +38,9 @@
 #ifdef HAVE_FAM
 #include <fam.h>
 #include <glib/giochannel.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 #endif
 #ifdef LIBGDA_WIN32
 #include <io.h>
@@ -68,7 +71,7 @@ typedef struct {
 	GList *user;   /* list of GdaConfigSection */
 } GdaConfigClient;
 
-static GdaConfigClient *config_client;
+static GdaConfigClient *config_client = NULL;
 static gboolean         can_modif_global_conf; /* TRUE if the current user can modify the system wide config file */
 static gboolean         lock_write_notify = FALSE;
 static void             do_notify (const gchar *path);
@@ -79,9 +82,12 @@ static void             do_notify (const gchar *path);
 #ifdef HAVE_FAM
 static FAMConnection *fam_connection = NULL;
 static gint           fam_watch_id = 0;
-gboolean              lock_fam = FALSE;
-FAMRequest           *fam_conf_user = NULL;
-FAMRequest           *fam_conf_global = NULL;
+static gboolean       lock_fam = FALSE;
+static FAMRequest    *fam_conf_user = NULL;
+static FAMRequest    *fam_conf_global = NULL;
+static time_t         last_mtime = 0;
+static time_t         last_ctime = 0;
+static off_t          last_size = 0;
 
 static gboolean       fam_callback (GIOChannel *source, GIOCondition condition, gpointer data);
 static void           fam_lock_notify ();
@@ -91,7 +97,7 @@ static void           fam_unlock_notify ();
 /*
  * Private functions
  */
-#ifdef GDA_DEBUG
+#ifdef GDA_DEBUG_NO
 static void
 dump_config_client ()
 {
@@ -276,14 +282,14 @@ get_config_client ()
 			file = fopen (LIBGDA_GLOBAL_CONFIG_FILE, "a");
 			if (file) {
 				can_modif_global_conf = TRUE;
-#ifdef GDA_DEBUG
+#ifdef GDA_DEBUG_NO
 				g_print ("Can write system wide config file\n");
 #endif
 				fclose (file);
 			}
 			else {
 				can_modif_global_conf = FALSE;
-#ifdef GDA_DEBUG
+#ifdef GDA_DEBUG_NO
 				g_print ("Cannot write system wide config file\n");
 #endif			
 			}
@@ -295,7 +301,7 @@ get_config_client ()
 			GIOChannel *ioc;
 			int res;
 	
-#ifdef GDA_DEBUG
+#ifdef GDA_DEBUG_NO
 			g_print ("Using FAM to monitor configuration files changes.\n");
 #endif
 			fam_connection = g_malloc0 (sizeof (FAMConnection));
@@ -313,7 +319,7 @@ get_config_client ()
 				fam_conf_global = g_new0 (FAMRequest, 1);
 				res = FAMMonitorFile (fam_connection, LIBGDA_GLOBAL_CONFIG_FILE, fam_conf_global, 
 						      GINT_TO_POINTER (TRUE));
-#ifdef GDA_DEBUG
+#ifdef GDA_DEBUG_NO
 				g_print ("Monitoring changes on file %s: %s\n", 
 					 LIBGDA_GLOBAL_CONFIG_FILE, res ? "ERROR" : "Ok");
 #endif
@@ -322,7 +328,7 @@ get_config_client ()
 					fam_conf_user = g_new0 (FAMRequest, 1);
 					res = FAMMonitorFile (fam_connection, user_config, fam_conf_user, 
 							      GINT_TO_POINTER (FALSE));
-#ifdef GDA_DEBUG
+#ifdef GDA_DEBUG_NO
 					g_print ("Monitoring changes on file %s: %s\n", 
 						 user_config, res ? "ERROR" : "Ok");
 #endif
@@ -434,7 +440,7 @@ get_config_client ()
 				g_warning ("Config file is not readable.");
 		}
 		g_free (user_config);
-#ifdef GDA_DEBUG
+#ifdef GDA_DEBUG_NO
 		dump_config_client ();
 #endif
 	}
@@ -468,10 +474,23 @@ fam_callback (GIOChannel *source, GIOCondition condition, gpointer data)
 
 		is_global = GPOINTER_TO_INT (ev.userdata);
 		switch (ev.code) {
-		case FAMChanged:
+		case FAMChanged: {
+			struct stat stat;
+			if (lstat (ev.filename, &stat))
+				break;
+			if ((stat.st_mtime != last_mtime) ||
+			    (stat.st_ctime != last_ctime) ||
+			    (stat.st_size != last_size)) {
+				last_mtime = stat.st_mtime;
+				last_ctime = stat.st_ctime;
+				last_size = stat.st_size;
+			}
+			else
+				break;
+		}
 		case FAMDeleted:
 		case FAMCreated:
-#ifdef GDA_DEBUG
+#ifdef GDA_DEBUG_NO
 			g_print ("Reloading config files (%s config has changed)\n", is_global ? "global" : "user");
 #endif
 			gda_config_client_reset ();
@@ -868,7 +887,7 @@ gda_config_get_boolean (const gchar *path)
  *
  * Sets the given configuration entry to contain a string.
  *
- * Returns: TRUE if no error occured
+ * Returns: TRUE if no error occurred
  */
 gboolean
 gda_config_set_string (const gchar *path, const gchar *new_value)
@@ -922,7 +941,7 @@ gda_config_set_string (const gchar *path, const gchar *new_value)
  *
  * Sets the given configuration entry to contain an integer.
  *
- * Returns: TRUE if no error occured
+ * Returns: TRUE if no error occurred
  */
 gboolean
 gda_config_set_int (const gchar *path, gint new_value)
@@ -977,7 +996,7 @@ gda_config_set_int (const gchar *path, gint new_value)
  *
  * Sets the given configuration entry to contain a float.
  *
- * Returns: TRUE if no error occured
+ * Returns: TRUE if no error occurred
  */
 gboolean
 gda_config_set_float (const gchar * path, gdouble new_value)
@@ -1032,7 +1051,7 @@ gda_config_set_float (const gchar * path, gdouble new_value)
  *
  * Sets the given configuration entry to contain a boolean.
  *
- * Returns: TRUE if no error occured
+ * Returns: TRUE if no error occurred
  */
 gboolean
 gda_config_set_boolean (const gchar *path, gboolean new_value)
@@ -1466,19 +1485,28 @@ gda_config_get_provider_list (void)
 				GError *error = NULL;
 				
 				info->dsn_spec = plugin_get_dsn_spec ();
-				info->gda_params = gda_parameter_list_new_from_spec (NULL, info->dsn_spec, &error);
-				if (!info->gda_params) {
-					g_warning ("Invalid format for provider '%s' DSN spec : %s",
-						   info->id,
-						   error ? error->message : "Unknown error");
-					if (error)
-						g_error_free (error);
+				if (info->dsn_spec) {
+					info->gda_params = gda_parameter_list_new_from_spec_string (NULL, info->dsn_spec, &error);
+					if (!info->gda_params) {
+						g_warning ("Invalid format for provider '%s' DSN spec : %s",
+							   info->id,
+							   error ? error->message : "Unknown error");
+						if (error)
+							g_error_free (error);
+					}
+				}
+				else {
+					/* there may be traces of the provider installed but some parts are missing,
+					 forget about that provider... */
+					gda_provider_info_free (info);
+					info = NULL;
 				}
 			}
 			else 
 				g_warning ("Provider '%s' does not provide a DSN spec", info->id);
 			
-			list = g_list_append (list, info);
+			if (info)
+				list = g_list_append (list, info);
 			
 			g_module_close (handle);
 		}
@@ -1574,12 +1602,21 @@ gda_config_get_provider_model (void)
 	for (l = prov_list; l != NULL; l = l->next) {
 		GList *value_list = NULL;
 		GdaProviderInfo *prov_info = (GdaProviderInfo *) l->data;
+		GValue *value;
 
 		g_assert (prov_info != NULL);
 
-		value_list = g_list_append (value_list, gda_value_new_string (prov_info->id));
-		value_list = g_list_append (value_list, gda_value_new_string (prov_info->location));
-		value_list = g_list_append (value_list, gda_value_new_string (prov_info->description));
+		value = g_value_init (g_new0 (GValue, 1), G_TYPE_STRING);
+		g_value_set_string (value, prov_info->id);
+		value_list = g_list_append (value_list, value);
+
+		value = g_value_init (g_new0 (GValue, 1), G_TYPE_STRING);
+		g_value_set_string (value, prov_info->location);
+		value_list = g_list_append (value_list, value);
+
+		value = g_value_init (g_new0 (GValue, 1), G_TYPE_STRING);
+		g_value_set_string (value, prov_info->description);
+		value_list = g_list_append (value_list, value);
 
 		gda_data_model_append_values (GDA_DATA_MODEL (model), value_list, NULL);
 		g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
@@ -1726,7 +1763,7 @@ gda_config_find_data_source (const gchar *name)
 	for (l = dsnlist; l != NULL; l = l->next) {
 		GdaDataSourceInfo *tmp_info = (GdaDataSourceInfo *) l->data;
 
-		if (tmp_info && !g_strcasecmp (tmp_info->name, name)) {
+		if (tmp_info && !g_ascii_strcasecmp (tmp_info->name, name)) {
 			info = gda_data_source_info_copy (tmp_info);
 			break;
 		}
@@ -1845,9 +1882,8 @@ gda_config_free_data_source_list (GList *list)
  * Fills and returns a new #GdaDataModel object using information from all 
  * data sources which are currently configured in the system.
  *
- * Rows are separated in 7 columns: 
- * 'Name', 'Provider', 'Connection string', 'Description', 'Username',
- * 'Password' and 'Global'.
+ * Rows are separated in 6 columns: 
+ * 'Name', 'Provider', 'Connection string', 'Description', 'Username' and 'Global'.
  *
  * Returns: a new #GdaDataModel object. 
  */
@@ -1858,30 +1894,46 @@ gda_config_get_data_source_model (void)
 	GList *l;
 	GdaDataModel *model;
 
-	model = gda_data_model_array_new (7);
+	model = gda_data_model_array_new (6);
 	gda_data_model_set_column_title (model, 0, _("Name"));
 	gda_data_model_set_column_title (model, 1, _("Provider"));
 	gda_data_model_set_column_title (model, 2, _("Connection string"));
 	gda_data_model_set_column_title (model, 3, _("Description"));
 	gda_data_model_set_column_title (model, 4, _("Username"));
-	gda_data_model_set_column_title (model, 5, _("Password"));
-	gda_data_model_set_column_title (model, 6, _("Global"));
+	gda_data_model_set_column_title (model, 5, _("Global"));
 	
 	/* convert the returned GList into a GdaDataModelArray */
 	dsn_list = gda_config_get_data_source_list ();
 	for (l = dsn_list; l != NULL; l = l->next) {
 		GList *value_list = NULL;
 		GdaDataSourceInfo *dsn_info = (GdaDataSourceInfo *) l->data;
+		GValue *value;
 
 		g_assert (dsn_info != NULL);
 
-		value_list = g_list_append (value_list, gda_value_new_string (dsn_info->name));
-		value_list = g_list_append (value_list, gda_value_new_string (dsn_info->provider));
-		value_list = g_list_append (value_list, gda_value_new_string (dsn_info->cnc_string));
-		value_list = g_list_append (value_list, gda_value_new_string (dsn_info->description));
-		value_list = g_list_append (value_list, gda_value_new_string (dsn_info->username));
-		value_list = g_list_append (value_list, gda_value_new_string ("******"));
-		value_list = g_list_append (value_list, gda_value_new_boolean (dsn_info->is_global));
+		value = g_value_init (g_new0 (GValue, 1), G_TYPE_STRING);
+		g_value_set_string (value, dsn_info->name);
+		value_list = g_list_append (value_list, value);
+
+		value = g_value_init (g_new0 (GValue, 1), G_TYPE_STRING);
+		g_value_set_string (value, dsn_info->provider);
+		value_list = g_list_append (value_list, value);
+
+		value = g_value_init (g_new0 (GValue, 1), G_TYPE_STRING);
+		g_value_set_string (value, dsn_info->cnc_string);
+		value_list = g_list_append (value_list, value);
+
+		value = g_value_init (g_new0 (GValue, 1), G_TYPE_STRING);
+		g_value_set_string (value, dsn_info->description);
+		value_list = g_list_append (value_list, value);
+
+		value = g_value_init (g_new0 (GValue, 1), G_TYPE_STRING);
+		g_value_set_string (value, dsn_info->username);
+		value_list = g_list_append (value_list, value);
+
+		value = g_value_init (g_new0 (GValue, 1), G_TYPE_BOOLEAN);
+		g_value_set_boolean (value, dsn_info->is_global);
+		value_list = g_list_append (value_list, value);
 		
 		gda_data_model_append_values (GDA_DATA_MODEL (model), value_list, NULL);
 		g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
@@ -1922,7 +1974,7 @@ gda_config_can_modify_global_config (void)
  * Adds a new data source (or update an existing one) to the GDA
  * configuration, based on the parameters given.
  *
- * Returns: TRUE if no error occured
+ * Returns: TRUE if no error occurred
  */
 gboolean
 gda_config_save_data_source (const gchar *name,
@@ -1993,13 +2045,15 @@ gda_config_save_data_source (const gchar *name,
 	g_assert (section);
 	section->is_global = is_global;
 
-	cfg_client->user = g_list_remove (cfg_client->user, section);
-	cfg_client->global = g_list_remove (cfg_client->global, section);
-
-	if (!g_list_find (cfg_client->global, section) && is_global)
+	if (section->is_global && (!g_list_find (cfg_client->global, section))) {
 		cfg_client->global = g_list_append (cfg_client->global, section);
-	if (!g_list_find (cfg_client->user, section) && !is_global)
+		cfg_client->user = g_list_remove (cfg_client->user, section);
+	}
+
+	if (!section->is_global && (!g_list_find (cfg_client->user, section))) {
 		cfg_client->user = g_list_append (cfg_client->user, section);
+		cfg_client->global = g_list_remove (cfg_client->global, section);
+	}
 
 	g_string_free (str, TRUE);
 
@@ -2019,7 +2073,7 @@ gda_config_save_data_source (const gchar *name,
  * #GdaDataSourceInfo structure containing all the information
  * about the data source.
  *
- * Returns: TRUE if no error occured
+ * Returns: TRUE if no error occurred
  */
 gboolean
 gda_config_save_data_source_info (GdaDataSourceInfo *dsn_info)
