@@ -1,6 +1,6 @@
 /* gda-query-target.c
  *
- * Copyright (C) 2003 - 2005 Vivien Malerba
+ * Copyright (C) 2003 - 2006 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -196,12 +196,14 @@ gda_query_target_class_init (GdaQueryTargetClass * class)
 	object_class->set_property = gda_query_target_set_property;
 	object_class->get_property = gda_query_target_get_property;
 	g_object_class_install_property (object_class, PROP_QUERY,
-					 g_param_spec_pointer ("query", "Query to which the target belongs", NULL, 
+					 g_param_spec_object ("query", "Query to which the target belongs", NULL, 
+                                                               GDA_TYPE_QUERY,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE |
 								G_PARAM_CONSTRUCT_ONLY)));
 
 	g_object_class_install_property (object_class, PROP_ENTITY_OBJ,
-					 g_param_spec_pointer ("entity", "A pointer to a GdaEntity", NULL, 
+					 g_param_spec_object ("entity", "A pointer to a GdaEntity", NULL, 
+                                                               GDA_TYPE_ENTITY,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_ENTITY_NAME,
 					 g_param_spec_string ("entity_name", "Name an entity", NULL, NULL,
@@ -236,7 +238,7 @@ gda_query_target_init (GdaQueryTarget *target)
  *
  * Returns: the new object
  */
-GObject *
+GdaQueryTarget *
 gda_query_target_new (GdaQuery *query, const gchar *table)
 {
 	GObject *obj;
@@ -247,7 +249,7 @@ gda_query_target_new (GdaQuery *query, const gchar *table)
 	obj = g_object_new (GDA_TYPE_QUERY_TARGET, 
 			    "dict", gda_object_get_dict (GDA_OBJECT (query)), 
 			    "query", query, "entity_name", table, NULL);
-	return obj;
+	return (GdaQueryTarget*) obj;
 }
 
 
@@ -259,7 +261,7 @@ gda_query_target_new (GdaQuery *query, const gchar *table)
  *
  * Returns: the new object
  */
-GObject *
+GdaQueryTarget *
 gda_query_target_new_copy (GdaQueryTarget *orig)
 {
 	GObject *obj;
@@ -290,7 +292,7 @@ gda_query_target_new_copy (GdaQueryTarget *orig)
 			gda_object_ref_set_ref_name (target->priv->entity_ref, ref_gtype, ref_type, ref_str);
 	}
 
-	return obj;	
+	return target;	
 }
 
 
@@ -358,7 +360,6 @@ gda_query_target_set_property (GObject *object,
 			       const GValue *value,
 			       GParamSpec *pspec)
 {
-	gpointer ptr;
 	GdaQueryTarget *target;
 	guint id;
 	const gchar *str;
@@ -367,8 +368,8 @@ gda_query_target_set_property (GObject *object,
 	target = GDA_QUERY_TARGET (object);
 	if (target->priv) {
 		switch (param_id) {
-		case PROP_QUERY:
-			ptr = g_value_get_pointer (value);
+		case PROP_QUERY: {
+			GdaQuery* ptr = GDA_QUERY (g_value_get_object (value));
 			g_return_if_fail (GDA_IS_QUERY (ptr));
 			g_return_if_fail (gda_object_get_dict (GDA_OBJECT (ptr)) == gda_object_get_dict (GDA_OBJECT (target)));
 			
@@ -389,11 +390,13 @@ gda_query_target_set_property (GObject *object,
 			g_object_get (G_OBJECT (ptr), "target_serial", &id, NULL);
 			gda_query_object_set_int_id (GDA_QUERY_OBJECT (target), id);
 			break;
-		case PROP_ENTITY_OBJ:
-			ptr = g_value_get_pointer (value);
+                }
+		case PROP_ENTITY_OBJ: {
+			GdaEntity* ptr = GDA_ENTITY (g_value_get_object (value));
 			g_return_if_fail (GDA_IS_ENTITY (ptr));
 			gda_object_ref_set_ref_object (target->priv->entity_ref, GDA_OBJECT (ptr));
 			break;
+                }
 		case PROP_ENTITY_NAME:
 			str = g_value_get_string (value);
 			gda_object_ref_set_ref_name (target->priv->entity_ref, GDA_TYPE_DICT_TABLE, 
@@ -425,10 +428,10 @@ gda_query_target_get_property (GObject *object,
 	if (target->priv) {
 		switch (param_id) {
 		case PROP_QUERY:
-			g_value_set_pointer (value, target->priv->query);
+			g_value_set_object (value, G_OBJECT (target->priv->query));
 			break;
 		case PROP_ENTITY_OBJ:
-			g_value_set_pointer (value, gda_object_ref_get_ref_object (target->priv->entity_ref));
+			g_value_set_object (value, G_OBJECT (gda_object_ref_get_ref_object (target->priv->entity_ref)));
 			break;
 		case PROP_ENTITY_NAME:
 		case PROP_ENTITY_ID:
@@ -879,6 +882,14 @@ gda_query_target_render_as_sql (GdaRenderer *iface, GdaParameterList *context, g
 	GdaQueryTarget *target;
 	GdaEntity *entity;
 	gboolean err = FALSE;
+	GdaServerProviderInfo *sinfo = NULL;
+	GdaDict *dict;
+	GdaConnection *cnc;
+	
+	dict = gda_object_get_dict (GDA_OBJECT (iface));
+	cnc = gda_dict_get_connection (dict);
+	if (cnc)
+		sinfo = gda_connection_get_infos (cnc);
 
 	g_return_val_if_fail (GDA_IS_QUERY_TARGET (iface), NULL);
 	g_return_val_if_fail (GDA_QUERY_TARGET (iface)->priv, NULL);
@@ -887,19 +898,19 @@ gda_query_target_render_as_sql (GdaRenderer *iface, GdaParameterList *context, g
 	entity = gda_query_target_get_represented_entity (target);
 	if (entity) {
 		if (GDA_IS_DICT_TABLE (entity)) {
-			gchar *tmp, *tname;
+			gchar *tmp = NULL, *tname;
 
 			tname = (gchar *) gda_object_get_name (GDA_OBJECT (entity));
-			tmp = g_utf8_strdown (tname, -1);
-			if (((*tmp <= '9') && (*tmp >= '0')) || 
-			    strcmp (tmp, tname)) {
-				g_free (tmp);
-				tmp = tname;
-				tname = g_strdup_printf ("\"%s\"", tmp);
+			if (!sinfo || sinfo->quote_non_lc_identifiers) {
+				tmp = g_utf8_strdown (tname, -1);
+				if (((*tmp <= '9') && (*tmp >= '0')) || strcmp (tmp, tname)) {
+					g_free (tmp);
+					tname = g_strdup_printf ("\"%s\"", tname);
+					tmp = tname;
+				}
 			}
-			else
-				g_free (tmp);
-			string = g_string_new (tname);
+			string = g_string_new (tname);			
+			g_free (tmp);
 		}
 		
 		if (GDA_IS_QUERY (entity)) {
@@ -928,16 +939,7 @@ gda_query_target_render_as_sql (GdaRenderer *iface, GdaParameterList *context, g
 	}
 
 	if (!err) {
-		/* adding alias */
-		GdaServerProviderInfo *sinfo = NULL;
-		GdaDict *dict;
-		GdaConnection *cnc;
-		
-		dict = gda_object_get_dict (GDA_OBJECT (target));
-		cnc = gda_dict_get_connection (dict);
-		if (cnc)
-			sinfo = gda_connection_get_infos (cnc);
-	
+		/* adding alias */	
 		if (!sinfo || sinfo->supports_alias) {
 			if (!sinfo || sinfo->alias_needs_as_keyword)
 				g_string_append (string, " AS ");
