@@ -165,13 +165,16 @@ gda_server_operation_class_init (GdaServerOperationClass *klass)
 	object_class->get_property = gda_server_operation_get_property;
 
 	g_object_class_install_property (object_class, PROP_DICT,
-					 g_param_spec_pointer ("dict", NULL, NULL, 
+					 g_param_spec_object ("dict", NULL, NULL, 
+                                                               GDA_TYPE_DICT,
 							       G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_CNC,
-					 g_param_spec_pointer ("connection", NULL, NULL, 
+					 g_param_spec_object ("connection", NULL, NULL, 
+                                                               GDA_TYPE_CONNECTION,
 							       G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_PROV,
-					 g_param_spec_pointer ("provider_obj", NULL, NULL, 
+					 g_param_spec_object ("provider_obj", NULL, NULL, 
+                                                               GDA_TYPE_SERVER_PROVIDER,
 							       G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_SPEC_FILE,
 					 g_param_spec_string ("spec_file", NULL, NULL, 
@@ -516,14 +519,14 @@ gda_server_operation_set_property (GObject *object,
 			if (op->priv->dict)
 				g_object_unref (op->priv->dict);
 
-			op->priv->dict = ASSERT_DICT (g_value_get_pointer (value));
+			op->priv->dict = ASSERT_DICT (g_value_get_object (value));
 			g_object_ref (op->priv->dict);
 			break;
 		case PROP_CNC:
 			if (op->priv->cnc)
 				g_object_unref (op->priv->cnc);
 
-			op->priv->cnc = g_value_get_pointer (value);
+			op->priv->cnc = GDA_CONNECTION( g_value_get_object (value));
 			op->priv->cnc_set = TRUE;
 
 			if (op->priv->cnc) {
@@ -539,10 +542,10 @@ gda_server_operation_set_property (GObject *object,
 			}
 			break;
 		case PROP_PROV:
-			if (g_value_get_pointer (value)) {
+			if (g_value_get_object (value)) {
 				if (op->priv->prov)
 					g_object_unref (op->priv->prov);
-				op->priv->prov = g_value_get_pointer (value);
+				op->priv->prov = g_value_get_object(value);
 				g_object_ref (op->priv->prov);
 			}
 			op->priv->prov_set = TRUE;
@@ -843,7 +846,7 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
 			/* find data type and create GdaParameter */
 			dbmstype = xmlGetProp (node, BAD_CAST "dbmstype");
 			gdatype = xmlGetProp (node, BAD_CAST "gdatype");
-			dtype = utility_find_or_create_data_type (op->priv->dict, op->priv->prov, op->priv->cnc,
+			dtype = gda_utility_find_or_create_data_type (op->priv->dict, op->priv->prov, op->priv->cnc,
 								  dbmstype, gdatype, &dtype_created);
 			if (dbmstype) xmlFree (dbmstype);
 			if (gdatype) xmlFree (gdatype);
@@ -861,7 +864,7 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
 					g_object_unref (dtype);
 			
 				/* set parameter's attributes */
-				utility_parameter_load_attributes (param, node, op->priv->sources);
+				gda_utility_parameter_load_attributes (param, node, op->priv->sources);
 
 				opnode = node_new (parent, GDA_SERVER_OPERATION_NODE_PARAM, path_name);
 				opnode->d.param = param;
@@ -1165,7 +1168,7 @@ node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent)
 	case GDA_SERVER_OPERATION_NODE_DATA_MODEL:
 		node = xmlNewChild (parent, NULL, BAD_CAST "op_data", NULL);
 		xmlSetProp (node, "path", complete_path);
-		utility_data_model_dump_data_to_xml (opnode->d.model, node, NULL, 0, NULL, 0, TRUE);
+		gda_utility_data_model_dump_data_to_xml (opnode->d.model, node, NULL, 0, NULL, 0, TRUE);
 		break;
 	case GDA_SERVER_OPERATION_NODE_PARAM: {
 		const GValue *value;
