@@ -159,7 +159,7 @@ test_and_destroy_dict (TestConfig *config)
 			GdaDict *loaded_dict;
 
 			/* g_print (_("Dictionary file written to temporary location %s\n"), dictfile); */
-			loaded_dict = GDA_DICT (gda_dict_new ());
+			loaded_dict = gda_dict_new ();
 			if (!gda_dict_load_xml_file (loaded_dict, dictfile, &error)) {
 				g_print (_("Error loading dictionary file '%s':\n%s\n"), dictfile,
 					 error ? error->message : _("Unspecified"));
@@ -261,7 +261,7 @@ test_models (TestConfig *config)
 			gchar *filename = xmlGetProp (subnode, "name");
 			test_and_destroy_dict (config);			
 			if (filename) {
-				config->dict = GDA_DICT (gda_dict_new ());
+				config->dict = gda_dict_new ();
 				g_print ("Loading dictionary %s\n", filename);
 				if (!gda_dict_load_xml_file (config->dict, filename, &error)) {
 					g_print ("Error occurred:\n\t%s\n", error->message);
@@ -380,30 +380,20 @@ action_load_file (TestConfig *config, const gchar *model_name, xmlNodePtr node,
 	gchar *file;
 	gchar *errmsg = NULL;
 
-	xmlDocPtr doc;
-	xmlNodePtr root;
-
 	file = xmlGetProp (node, "file");
 	if (!file)
 		errmsg = g_strdup (_("Missing 'file' attribute"));
 	else {
-		doc = xmlParseFile (file);
-		if (!doc) 
-			errmsg = g_strdup_printf (_("Can't load file '%s'"), file);
-		else {
-			GSList *errors;
+		GSList *errors;
 
-			import = gda_data_model_import_new_xml_node (xmlDocGetRootElement (doc));
-			errors = gda_data_model_import_get_errors (GDA_DATA_MODEL_IMPORT (import));
-			if (errors) {
-				GError *err = (GError *) errors->data;
-
-				errmsg = g_strdup (err && err->message ? err->message : _("Unknown error"));
-				g_object_unref (import);
-				import = NULL;
-			}
-
-			xmlFreeDoc (doc);
+		import = gda_data_model_import_new_file (file, FALSE, NULL);
+		errors = gda_data_model_import_get_errors (GDA_DATA_MODEL_IMPORT (import));
+		if (errors) {
+			GError *err = (GError *) errors->data;
+			
+			errmsg = g_strdup (err && err->message ? err->message : _("Unknown error"));
+			g_object_unref (import);
+			import = NULL;
 		}
 		g_free (file);
 	}
@@ -420,6 +410,7 @@ action_load_file (TestConfig *config, const gchar *model_name, xmlNodePtr node,
 			errmsg = g_strdup (error && error->message ? error->message : _("Unknown error"));
 			g_error_free (error);
 		}
+		g_object_unref (import);
 	}
 
 	handle_error (config, errmsg, expected_res);
@@ -538,16 +529,16 @@ action_set_value (TestConfig *config, GdaDataModel *model, xmlNodePtr node, gboo
 	if (!column)
 		errmsg = g_strdup_printf ("Can't find column %d", col);
 	else {
-		GdaValueType gdatype;
+		GType gdatype;
 
-		gdatype = gda_column_get_gda_type (column);
-		if ((gdatype == GDA_VALUE_TYPE_UNKNOWN) ||
-		    (gdatype == GDA_VALUE_TYPE_NULL)) {
+		gdatype = gda_column_get_g_type (column);
+		if ((gdatype == G_TYPE_INVALID) ||
+		    (gdatype == GDA_TYPE_NULL)) {
 			errmsg = g_strdup (_("Cannot retrieve column data type (type is UNKNOWN or not specified)"));
 		}
 		else {
 			gchar *isnull;
-			GdaValue *value = NULL;
+			GValue *value = NULL;
 
 			isnull = xmlGetProp (vnode, "isnull");
 			if (isnull) {
@@ -558,11 +549,11 @@ action_set_value (TestConfig *config, GdaDataModel *model, xmlNodePtr node, gboo
 			}
 
 			if (!isnull) {
-				value = g_new0 (GdaValue, 1);
+				value = g_new0 (GValue, 1);
 				if (!gda_value_set_from_string (value, xmlNodeGetContent (vnode), gdatype)) {
 					g_free (value);
 					errmsg = g_strdup_printf (_("Cannot interpret string as a valid %s value"), 
-								   gda_type_to_string (gdatype));
+								   gda_g_type_to_string (gdatype));
 				}
 			}
 			else
